(ns ratata.core
  (:require [quil.core :as q]
            [quil.middleware :as m]))

(defn setup []
  (q/frame-rate 10)
  (q/background 0 0 0)
  (q/fill 255 255 255)
  (q/stroke 250 250 250)
  (q/stroke-join :round)
  {:color (q/color 250 250 250), :time 0})

(defn update-state [{:keys [color time] :as state}]
  {:color color, :time (+ time 0.1)})

(defn draw-state [state]  
  (q/stroke (:color state))
  (def szx 12)
  (def szy 12)
  (def dx (/ (q/width) szx))
  (def dy (/ (q/height) szy))
  (doseq [x (range (+ szx 1)) y (range (+ 1 szy))]
    (def posx (* x dx))
    (def posy (* y dy))
    (def n (q/map-range
            (q/noise (/ (+ 1 x) 2) (/ (+ 1 y) 5) (:time state))
            0 1 (- (/ q/PI 5)) (/ q/PI 5)))
    ;; (def n (q/random (- (/ q/PI 10)) (/ q/PI 10)))
    (q/stroke-weight (+ (if (<= x (/ szx 2)) (* 2 x) (- szx x))
                        1))
    (q/with-translation [posx posy]
      (q/with-rotation [(- (* x (/ q/PI (+ 1 szx))) n)]
        (q/line 0 0 dx 0)))))

;; magic middleware
(defn trnp [n odraw state]
  (q/background 0 0 0)
  (doseq [i (range 0 n)]
    (q/with-translation [(* 10 i) 0]
      (odraw (state)))))

(defn trnp-me [options]
  (let [draw (:draw options (fn [state]))] 
    (assoc options :draw (partial trnp 2 draw))))

(q/defsketch ratata
  :title "RATATA"
  :size [900 700]
  ; setup function called only once, during sketch initialization.
  :setup setup
  ; update-state is called on each iteration before draw-state.
  :update update-state
  :draw draw-state
  :features [:keep-on-top]
  ; This sketch uses functional-mode middleware.
  ; Check quil wiki for more info about middlewares and particularly
  ; fun-mode.
  :middleware [m/fun-mode, trnp-me])
